/*
**  WASABI-Hot! version 1.2b   
**
**
**      -- copyright (c) 2001-2003 by Philips Japan, Ltd. -- All rights reserved --
**
**
**      ** This code has been made to check/learn                          ** 
**      **                             the ISP1362/ISP1363 functionalities **
**      ** Release 08-Dec-2003                                             **
**
**      OKANO, Akifumi
**      
**		Application Laboratory, Portable Systems
**      Semiconductors Div, Philips Japan Ltd.
**      akifumi.okano@philips.com
**      +81-3-3740-4668
*/


/****************************************************************************/
/*	includes																*/
/****************************************************************************/

#include		<stdio.h>
#include		<stdlib.h>
#include		<alloc.h>
#include		<string.h>
#include		<ctype.h>
#include		<time.h>

#include		"general.h"
#include		"_hc_core/dev_ep.h"

#include		"class_dr/storage/fs.h"
#include		"class_dr/storage/fs_shell.h"
#include		"class_dr/storage/storage.h"

#include		"_hc_cls/cls_hndl.h"

#include		"ui.h"


/****************************************************************************/
/*	constants																*/
/****************************************************************************/

#define			COMM_STR_BUFFER_SIZE			16
#define			MAX_COMMAND_TOKENS				30

#if 0
enum FSTS_COLORS {
    FSTS_BLACK         = (LIGHTGRAY << 4),	// dark colors
    FSTS_BLUE,
    FSTS_GREEN,
    FSTS_CYAN,
    FSTS_RED,
    FSTS_MAGENTA,
    FSTS_BROWN,
    FSTS_LIGHTGRAY,
    
    FSTS_DARKGRAY,       // light colors
    FSTS_LIGHTBLUE,
    FSTS_LIGHTGREEN,
    FSTS_LIGHTCYAN,
    FSTS_LIGHTRED,
    FSTS_LIGHTMAGENTA,
    FSTS_YELLOW,
    FSTS_WHITE
};
#define		FSTS_NORM	FSTS_BLACK

#else
enum FSTS_COLORS {
    FSTS_BLACK         = 0,	// dark colors
    FSTS_BLUE,
    FSTS_GREEN,
    FSTS_CYAN,
    FSTS_RED,
    FSTS_MAGENTA,
    FSTS_BROWN,
    FSTS_LIGHTGRAY,
    
    FSTS_DARKGRAY,       // light colors
    FSTS_LIGHTBLUE,
    FSTS_LIGHTGREEN,
    FSTS_LIGHTCYAN,
    FSTS_LIGHTRED,
    FSTS_LIGHTMAGENTA,
    FSTS_YELLOW,
    FSTS_WHITE
};
#define		FSTS_NORM	FSTS_LIGHTGRAY
#endif



/****************************************************************************/
/*	global vars																*/
/****************************************************************************/

unsigned char	g_file_system_test_shell_ON		= False;

unsigned char	g_comm_str_buffer_last			= 0;
unsigned char	g_comm_str_buffer_start			= 0;
unsigned char	g_comm_str_buffer_current		= 0;

char			g_comm_str_buffer[ COMM_STR_BUFFER_SIZE ][ MAX_PATH_STRING_LENGTH ]
															=	{
																	"", "", "", "",
																	"", "", "", "",
																	"", "", "", "",
																	"", "", "", ""
																};

/****************************************************************************/
/*	function prototypes														*/
/****************************************************************************/

static void				go_into_fsts( void );
static void				back_from_fsts( void );
static void				file_system_test_shell_process( void );
static void				fsts_ls( char *path );
static void				format_file_name_for_screen( char *str, char *filename, char *suffix );
static unsigned char	show_file_entry( directory_entry *entry_ptr );
static void				fsts_cd( char *path );
static void				fsts_rm( char **path, unsigned short n, unsigned char recursive );
static void				fsts_mv( char *from, char *to );
static void				fsts_mkdir( char **path, unsigned short n );
static void				fsts_df( void );
static unsigned char	split_command_string( char **token, char *string );
static void				fsts_get( char *source_name, char *target_name );
static void				fsts_put( char *source_name, char *target_name );
static void				fsts_cp( char *source_name, char *target_name );
static void				fsts_cs( char *str );
static void				fsts_fat( char *str );
static void				fsts_format( void );
static void				show_fsts_help( void );
static unsigned short 	storage_play_fill( short *buffer_ptr );
static void				storage_play_stop( void );
void					fsts_stop( void );

static void				fsts_test( void );
static void				fsts_test1( char *source_name, char *target_name );
static void				fsts_make1mb_local( char *target_name );

/****************************************************************************/
/*	function definitions													*/
/****************************************************************************/

//	FSTS

/*
==============================================================================
==============================================================================
====	Public function for file system access user interface
====		Function for APPLICATION side
==============================================================================
==============================================================================
*/

void fsts_file_system_test_shell( void )
{
	storage_mount();

	if ( NULL == storage_device_ready() )
	{
		mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nStorage device not ready.\r\n" );
		return;
	}

	if ( NULL == storage_volume_ready() )
	{
		mprintf( FSTS_LIGHTMAGENTA, CONTINUE, "\r\n  Started to mount a volume\r\n" );
		return;
	}
	
	go_into_fsts();
}


/*
==============================================================================
==============================================================================
====	Private functions for file system access user interface
====		Functions
==============================================================================
==============================================================================
*/

static void go_into_fsts( void )
{
	unsigned char	i;

	gene_install_asynchronous_periodic_process( 6, file_system_test_shell_process );
	g_file_system_test_shell_ON		= True;
	ui_show_current_time( False );
	mprintf( (RED<<4) | WHITE, CONTINUE, "\r\nUser interface switched into file system test shell" );
	mprintf( BLACK, CONTINUE, "\r\n" );
}


static void back_from_fsts( void )
{
	unsigned char	i;

	gene_install_asynchronous_periodic_process( 6, gene_maintenance_process );
	g_file_system_test_shell_ON		= False;
	ui_show_current_time( True );
	mprintf( (RED<<4) | WHITE, CONTINUE, "\r\nUser interface come backed from file system test shell" );
	mprintf( BLACK, CONTINUE, "\r\n" );
}


static void file_system_test_shell_process( void )
{
	static unsigned short	is_in_execution		= False;
	char					comm_str[ MAX_PATH_STRING_LENGTH ]	= "  ";
	char					prompt[ MAX_PATH_STRING_LENGTH + 10 ];
	char					pwd[ MAX_PATH_STRING_LENGTH ];
	char					*token[ MAX_COMMAND_TOKENS ];
	unsigned char			n;
	unsigned short			key;

	if ( is_in_execution )
		return;
	else
		is_in_execution		= True;

	if ( (NULL == storage_device_ready()) || (NULL == storage_volume_ready()) )
		back_from_fsts();
		
	if ( ui_key_polling_timing() )	//	Get timing to sample keyboard hit
	{
		//	Process user command

		//	
		//	Showing prompt
		//
		mprintf( FSTS_WHITE, CONTINUE, "\r                                      " );
		
		fs_pwd( pwd );
		sprintf( prompt, "[%u][%s] fsts% ", storage_device_address(), pwd );
		ui_get_string_from_console( comm_str, MAX_PATH_STRING_LENGTH, FSTS_NORM, prompt, FSTS_LIGHTCYAN, 0, g_comm_str_buffer[ g_comm_str_buffer_current ], &key );

		if ( (NULL == storage_device_ready()) || (NULL == storage_volume_ready()) )
			back_from_fsts();
			
		//	
		//	Browsing history by up/down arrow key
		//
		if ( !(key & 0x00FF) )
		{
			if ( key == 0x4800 )
			{
				if (g_comm_str_buffer_current != g_comm_str_buffer_start)
					g_comm_str_buffer_current		= --g_comm_str_buffer_current & 0x0F;

				mprintf( FSTS_WHITE, CONTINUE, "\r" );
				is_in_execution		= False;
				return;
			}
			else if ( key == 0x5000 )
			{
				if (g_comm_str_buffer_current != g_comm_str_buffer_last)
					g_comm_str_buffer_current		= ++g_comm_str_buffer_current & 0x0F;

				mprintf( FSTS_WHITE, CONTINUE, "\r" );
				is_in_execution		= False;
				return;
			}
		}
		
		
		//	
		//	Storing command to history
		//
		if ( *comm_str != 'e' )
		{
			fs_data_copy( g_comm_str_buffer[ g_comm_str_buffer_last ], comm_str, MAX_PATH_STRING_LENGTH );
			g_comm_str_buffer_last		= ++g_comm_str_buffer_last & 0x0F;

			if ( g_comm_str_buffer_last == g_comm_str_buffer_start )
				g_comm_str_buffer_start		= ++g_comm_str_buffer_start & 0x0F;

			g_comm_str_buffer_current	= g_comm_str_buffer_last;

			*(g_comm_str_buffer[ g_comm_str_buffer_last ])	= 0;
		}

		//	
		//	Parsing command
		//
		if ( *comm_str == 'e' )
			back_from_fsts();
		
		else if (*comm_str == '?')
			show_fsts_help();

		else if (*comm_str == 'i')
			storage_info();

		else if (*comm_str == 'i')
			storage_info();

		else if (*comm_str == 'h')
		{
			unsigned char	i	= g_comm_str_buffer_start;
			
			while ( i != g_comm_str_buffer_last )
			{
				mprintf( FSTS_NORM, CONTINUE, "\r\n    %s", g_comm_str_buffer[ i ] );
				i++;
				i	&= 0xF;
			}
		}
		else
		{
			n	= split_command_string( token, comm_str );

			if ( !strncmp( "ls", token[ 0 ], 2) )
			{
				if ( n == 1 )
					fsts_ls( "" );
				else if ( n == 2 )
					fsts_ls( token[ 1 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}		
			else if ( !strncmp( "cd", token[ 0 ], 2) )
			{
				if ( n == 2 )
					fsts_cd( token[ 1 ] );
				else if ( n == 1 )
				{
					mprintf( FSTS_LIGHTRED, CONTINUE, " -> pwd : " );
					mprintf( FSTS_WHITE, CONTINUE, "\r\n%s", pwd );
				}
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "check number of arguments" );			
			}
			else if ( !strncmp( "rmr", token[ 0 ], 3) )
			{
				fsts_rm( token, n, True );
			}
			else if ( !strncmp( "rm", token[ 0 ], 2) )
			{
				fsts_rm( token, n, False );
			}
			else if ( !strncmp( "mv", token[ 0 ], 2) )
			{
				if ( n == 3 )
					fsts_mv( token[ 1 ], token[ 2 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}		
			else if ( !strncmp( "mkdir", token[ 0 ], 5) )
			{
				if ( 1 < n )
					fsts_mkdir( token, n );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}		
			else if ( !strncmp( "df", token[ 0 ], 2) )
			{
				if ( 1 == n )
					fsts_df();
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}		
			else if ( !strncmp( "pwd", token[ 0 ], 3) )
			{
				mprintf( FSTS_WHITE, CONTINUE, "\r\n%s\r\n", pwd );
			}	
			else if ( !strncmp( "get", token[ 0 ], 3) )
			{
				if ( n == 2 )
					fsts_get( token[ 1 ], token[ 1 ] );
				else if ( n == 3 )
					fsts_get( token[ 1 ], token[ 2 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}		
			else if ( !strncmp( "put", token[ 0 ], 3) )
			{
				if ( n == 2 )
					fsts_put( token[ 1 ], token[ 1 ] );
				else if ( n == 3 )
					fsts_put( token[ 1 ], token[ 2 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}
			else if ( !strncmp( "cp", token[ 0 ], 2) )
			{
				if ( n == 3 )
					fsts_cp( token[ 1 ], token[ 2 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );
			}
			else if ( !strncmp( "play", token[ 0 ], 4) )
			{
				if ( n == 2 )
					fsts_play( token[ 1 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "stop", token[ 0 ], 4) )
			{
				if ( n == 1 )
					fsts_stop();
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "dump", token[ 0 ], 4) )
			{
				if ( n == 1 )
					storage_dump_info( 0, 1024 );
				else if ( n == 2 )
					storage_dump_info( atol( token[ 1 ] ), 0 );
				else if ( n == 3 )
					storage_dump_info( atol( token[ 1 ] ), atol( token[ 2 ] ) );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "cs", token[ 0 ], 2) )
			{
				if ( n == 2 )
					fsts_cs( token[ 1 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "fat", token[ 0 ], 3) )
			{
				if ( n == 2 )
					fsts_fat( token[ 1 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "format", token[ 0 ], 6) )
			{
				if ( n == 1 )
					fsts_format();
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "test", token[ 0 ], 4) )
			{
				if ( n == 1 )
					fsts_test();
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}
			else if ( !strncmp( "1mb", token[ 0 ], 3) )
			{
				if ( n == 1 )
					fsts_make1mb_local( "test1mb.bin" );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
			}


			else if ( !strncmp( "unmount", token[ 0 ], 7) )
			{
				if ( storage_remount( False ) )
					mprintf( FSTS_LIGHTRED,  CONTINUE, "\r\nCan not unmount\r\n" );
			}
			else if ( !strncmp( "mount", token[ 0 ], 5) )
			{
				if ( storage_mount() )
					mprintf( FSTS_LIGHTRED,  CONTINUE, "\r\nCan not mount\r\n" );
			}
			else if ( !strncmp( "read1", token[ 0 ], 5) )
			{
				if ( n == 2 )
					fsts_test1( token[ 1 ], token[ 1 ] );
				else
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments (%u)", n );			
			}
			else
			{
				mprintf( FSTS_LIGHTRED,  CONTINUE, "\r\nunknown command  " );
				mprintf( FSTS_NORM, CONTINUE, ": type [e] (for exit) and [return] to quit from FSTS.\r\n" );
			}
		}

		mprintf( FSTS_NORM, CONTINUE, "\r\n" );
		
		
		//	Update screen info
		
		ui_status_monitor();
		
		// Poll the event (for topology change event)
		
		portctl_isr_flag_routine_dspatch();
	}

	if ( is_in_execution )
		is_in_execution		= False;
}


static unsigned char split_command_string( char **token, char *string )
{
	unsigned short	i;
	char			c;
	char			*s;
	unsigned short	n		= 0;
	
	s	= string;
	
	while ( (s - string) < MAX_PATH_STRING_LENGTH )
	{
		while ( isspace( *s++ ) )
			;
	
		s--;
		
		if ( !(*s) )
			return ( n );
			
		*(token + n)	= s;
		n++;

		if ( n == MAX_COMMAND_TOKENS )
			return ( n );

		while ( !isspace( c	= *s++ ) )
			if ( !c )
				return ( n );

		*(s - 1)	= 0;
	}
	
	return ( n );
}


static void fsts_ls( char *path )
{
	directory_entry		*dp;
	unsigned short		entries;
	unsigned short		valid;
	unsigned short		i;

	unsigned short		available_cluster;
	unsigned short		total_cluster;
	unsigned short		cluster_size;

	unsigned char		first_char;

	entries		= 512;

	if ( NULL == (dp	= (directory_entry *)malloc( sizeof( directory_entry ) * entries )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "error : malloc @ ls command\r\n" );
		return;
	}
	
	if ( fs_ls( path, dp, entries ) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nno directory \"%s\"\r\n", path );
		free( dp );
		return;
	}

	mprintf( FSTS_NORM, CONTINUE, "\r\nList of directory\r\n" );
	mprintf( FSTS_NORM, CONTINUE, "    attr    file name     date        time            size    cluster\r\n" );

	valid	= 0;
	
	for ( i = 0; i < entries; i++ )
	{
		first_char	= *((unsigned char *)(dp + i));
	
		if ( !first_char )
			break;
		
		if ( (first_char == 0xE5) || ((dp + i)->attributes & 0x8) )
			continue;

		show_file_entry( dp + i );
		
		valid++;
		
		if ( !((valid + 1) % 8) )
		{
			mprintf( FSTS_LIGHTRED, CONTINUE, "    Press key to continue. Key [esc] to break.\r" );
			if ( (ui_key_wait() & 0xFF) == 0x1B )
			{
				mprintf( FSTS_LIGHTRED, CONTINUE, "                                                \r" );
				free( dp );
				return;
			}
		}
	}
#if 1
	mprintf( FSTS_NORM, CONTINUE, "    valid %u / total %u entries in this directory\r\n", valid, i );
	mprintf( FSTS_RED, CONTINUE, "    Calculating free space.\r" );
	
	fs_df( &available_cluster, &total_cluster, &cluster_size );

	mprintf( FSTS_NORM, CONTINUE, "    %6.2lf%%", ((double)(total_cluster - available_cluster) / (double)total_cluster) * 100.00 );
	mprintf( FSTS_NORM, CONTINUE, " of disk space used. %luK bytes free\r\n", ((unsigned long)available_cluster) * ((unsigned long)cluster_size) >> 10 );
#endif //DEBUG
	

	free( dp );
}


static void format_file_name_for_screen( char *str, char *filename, char *suffix )
{
	unsigned char	i;
	
	for ( i = 0; i < 8; i++ )
		*str++	= tolower( *filename++ );
	
	*str++	= '.';

	for ( i = 0; i < 3; i++ )
		*str++	= tolower( *suffix++ );

	*str	= 0;
}


static unsigned char show_file_entry( directory_entry *entry_ptr )
{
	char			filename[ 13 ];
	static char		*month_name[]		=	{ "???", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	static char		attrib_char[]		=	{ 'r', 'h', 's', 'v', 'd', 'a' };
	unsigned char	color;
	char			i;

#if 0
	if ( !(*((unsigned char *)entry_ptr)) )
		return ( 1 );
	
	if ( *((unsigned char *)entry_ptr) == 0xE5 )	//	deleted entry
		return ( 0 );
		
	if ( entry_ptr->attributes & 0x08 )				//	This means volume label. But how can I handle this?
		return ( 0 );
#endif
	
	format_file_name_for_screen( filename, entry_ptr->filename, entry_ptr->suffix );
	
	if      ( entry_ptr->attributes & 0x10 )
		color	= FSTS_YELLOW;
	else if ( entry_ptr->attributes & 0x06 )
		color	= FSTS_NORM;
	else
		color	= FSTS_WHITE;

	mprintf( color, CONTINUE, "    ", entry_ptr->attributes );

	for ( i = 5; i >= 0; --i )
	{
		if ( (entry_ptr->attributes >> i) & 0x1 )
			mprintf( color, CONTINUE, "%c", attrib_char[ i ] );
		else
			mprintf( color, CONTINUE, "-" );
	}
	mprintf( color, CONTINUE, "   ", entry_ptr->attributes );
				
	mprintf( color, CONTINUE, 
				"%s  %04d-%s-%02d %02d:%02d:%02d  %10ld     0x%04X\r\n", 
					filename, 
					
					((entry_ptr->date) >>  9) + 1980, 
					month_name[ ( (entry_ptr->date) >>  5) & 0x000F ], 
					((entry_ptr->date) >>  0) & 0x001F, 
					
					((entry_ptr->time) >> 11), 
					((entry_ptr->time) >>  5) & 0x003F, 
					((entry_ptr->time) <<  1) & 0x001F, 
					
					entry_ptr->size, 
					entry_ptr->cluster
			);

	return ( 0 );
}




static void fsts_cd( char *path )
{
	if ( fs_cd( path ) )
		mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ndirectory could not be found\r\n" );
}


static void fsts_rm( char **path, unsigned short n, unsigned char recursive )
{
	unsigned char	err;
	unsigned short 	i;
	
	if ( n == 1 )
	{
		mprintf( FSTS_LIGHTRED, CONTINUE, "\r\ncheck number of arguments" );			
		return;
	}
	
	for ( i = 1; i < n; i++ )
	{
		if ( 0 != (err	= fs_rm( *(path + i), recursive )) )
		{
			switch ( err )
			{
				case FS_RM_DIRECTORY_ACCESS_ERR :
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nrm error : FS_RM_DIRECTORY_ACCESS_ERR" );
					break;
				case FS_RM_SUB_DIRECTORY_EXIST :
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nrm error : FS_RM_SUB_DIRECTORY_EXIST" );
					break;
				default :
					mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nrm error : FS_RM_NOT_DIRECTORY_EMPTY" );
					break;
			}
		}
	}
}


static void fsts_mv( char *from, char *to )
{
	if ( fs_mv( from, to ) )
		mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nfile could not be moved\r\n" );
}


/*
void fsts_clean( char **path, unsigned short n )
{
	unsigned short 	i;
	
	for ( i = 1; i < n; i++ )
		if ( fs_rm( *(path + i) ) )
			mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nfile could not be found. file name : %s\r\n", *(path + i) );
}
*/

static void fsts_mkdir( char **path, unsigned short n )
{
	unsigned short 	i;
	
	for ( i = 1; i < n; i++ )
		if ( fs_mkdir( *(path + i) ) )
			mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nfile could not be found. file name : %s\r\n", *(path + i) );
}


static void fsts_df( void )
{
	unsigned short	available_cluster;
	unsigned long	available;
	unsigned short	total_cluster;
	unsigned long	total;
	unsigned short	cluster_size;
	
	fs_df( &available_cluster, &total_cluster, &cluster_size );
	
	available	= ((unsigned long)available_cluster * (unsigned long)cluster_size) >> 10L;
	total		= ((unsigned long)total_cluster     * (unsigned long)cluster_size) >> 10L;
	
	mprintf( FSTS_NORM, CONTINUE, "\r\n       total(KB)       used(KB)   available(KB)       capacity\r\n", total, total-available, available, cluster_size );
	mprintf( FSTS_NORM, CONTINUE, "       %9lu      %9lu       %9lu         %5.1lf%%\r\n", total, total - available, available, ((double)(total - available) / (double)total) * 100.00, cluster_size );
}


unsigned short	debug_error	= 0;
unsigned char check_file_content( char *orig, char *copied );

static void fsts_test( void )
{
	unsigned short	i;
	unsigned short	prev_error;
	
	for ( i = 0; i < 100; i++ )
	{
		mprintf( WHITE, CONTINUE, " ====== TEST === TEST === TEST ======     ROUND:%u (0-99))\r",i );
		
		prev_error	= debug_error;
		
//		fsts_format();
		fsts_put( "gymno1.zip", "gymno1.zip" );
		fsts_get( "gymno1.zip", "got.zip" );
		
		mprintf( WHITE, CONTINUE, "\r\nput-get done\r\n",  prev_error, debug_error );

		if ( prev_error != debug_error )
		{
			mprintf( WHITE, CONTINUE, "error happened while put\r\n" );
			break;
		}
		
		if ( check_file_content( "gymno1.zip", "got.zip" ) )
			break;
		
		storage_show_cclog_result();
	}

	if ( i == 100 )
		mprintf( (WHITE << 4), CONTINUE, "\r\nTEST completed with no problem\r\n" );
	else
		mprintf( (LIGHTRED << 4), CONTINUE, "\r\nTEST aborted with error (%u)\r\n", i );

	storage_show_cclog_result();
}


unsigned char check_file_content( char *orig, char *copied )
{
	FILE			*fo;
	FILE			*fc;
	unsigned char	bo[ 10240 ];
	unsigned char	bc[ 10240 ];
	unsigned short	orig_sz;
	unsigned long	proc_sz		= 0;
	unsigned short	i;


	if ( NULL == (fo	= fopen( orig, "rb" )) )
	{
		mprintf( WHITE, CONTINUE, "can't open file \"%s\"\r\n", orig );
		return ( 1 );
	}
	
	if ( NULL == (fc	= fopen( copied, "rb" )) )
	{
		mprintf( WHITE, CONTINUE, "can't open file \"%s\"\r\n", copied );
		fclose( fo );
		return ( 1 );
	}
	
//	while ( orig_sz		= fread( bo, 1, 10240, fo ) )
	while ( 0 != (orig_sz		= fread( bo, 1, 10240, fo )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "file checking... %lu\r", proc_sz );

		if ( orig_sz != fread( bc, 1, 10240, fc ) )
		{
			mprintf( FSTS_NORM, CONTINUE, "file size doesn't match\r\n" );

			fclose( fo );
			fclose( fc );
			return ( 1 );
		}	
		for ( i = 0; i < orig_sz; i++ )
		{
//			if ( *bo++ != *bc++ )
			if ( bo[ i ] != bc[ i ] )
			{
				mprintf( FSTS_NORM, CONTINUE, "file contents doesn't match\r\n" );
				
				fclose( fo );
				fclose( fc );
				return ( 1 );
			}
		}
		
		proc_sz		+= orig_sz;
	}
	
	mprintf( FSTS_NORM, CONTINUE, "file checking... %lu\r\n", proc_sz );

	fclose( fo );
	fclose( fc );
	
	return ( 0 );
}


static void fsts_test1( char *source_name, char *target_name )
{
	fs_FILE			*fs_fp;
	unsigned char	bp[ RW_BUFFER_SIZE ];
	unsigned short	i;
	unsigned short	size;
	unsigned long	total	= 0;
	clock_t			t_meas;

	
	if ( NULL == (fs_fp	= fs_fopen( source_name, R )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : file open @ fs_fopen( \"%s\", R )\r\n", source_name );
		return;
	}
	
	mprintf( FSTS_NORM, CONTINUE, "\r\nreading file USB:\"%s\" >> local:\"%s\"\r\n", source_name, target_name );

	t_meas	= clock();
	
//	while ( size	= fs_read( fs_fp, bp, RW_BUFFER_SIZE ) )
	while ( 0 != (size	= fs_read( fs_fp, bp, RW_BUFFER_SIZE )) )
		total += size;
			
	t_meas	= clock() - t_meas;
	
	if ( t_meas )
		mprintf( FSTS_NORM, CONTINUE, "read        %ld bytes     %.2f sec     %.2f B/sec\r", total += size, (double)t_meas / (double)CLK_TCK, (double)total / ((double)t_meas / (double)CLK_TCK) );

	fs_fclose( fs_fp );
}



static void fsts_make1mb_local( char *target_name )
{
	FILE			*fo;	
	unsigned char	bp[ 1024 ];
	unsigned short	i;
	unsigned short	size;

	for ( i = 0; i < 1024; i++ )
		bp[ i ]		= (unsigned char)i;
		
	if ( NULL == (fo	= fopen( target_name, "wb" )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : local file open to write : \"%s\"\r\n", target_name );
		return;
	}

	for ( i = 0; i < 1024; i++ )
		fwrite( bp, 1, 1024, fo );
	
	fclose( fo );
}



static void fsts_get( char *source_name, char *target_name )
{
	FILE			*fo;	
	fs_FILE			*fs_fp;
	unsigned char	bp[ RW_BUFFER_SIZE ];
	unsigned short	i;
	unsigned short	size;
	unsigned long	total	= 0;
	clock_t			t_meas;

	
	if ( NULL == (fo	= fopen( target_name, "wb" )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : local file open to write : \"%s\"\r\n", target_name );
		return;
	}

	if ( NULL == (fs_fp	= fs_fopen( source_name, R )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : file open @ fs_fopen( \"%s\", R )\r\n", source_name );
		fclose( fo );
		return;
	}
	
	mprintf( FSTS_NORM, CONTINUE, "\r\ngetting file USB:\"%s\" >> local:\"%s\"\r\n", source_name, target_name );

	t_meas	= clock();
	
//	while ( size	= fs_read( fs_fp, bp, RW_BUFFER_SIZE ) )
	while ( 0 != (size	= fs_read( fs_fp, bp, RW_BUFFER_SIZE )) )
	{
		fwrite( bp, 1, size, fo );
		mprintf( FSTS_NORM, CONTINUE, "reading...  %ld bytes\r", total += size );
		wait_ms( 0 );
	}
	
	t_meas	= clock() - t_meas;
	
	if ( t_meas )
		mprintf( FSTS_NORM, CONTINUE, "read        %ld bytes     %.2f sec     %.2f B/sec\r", total += size, (double)t_meas / (double)CLK_TCK, (double)total / ((double)t_meas / (double)CLK_TCK) );

	fclose( fo );
	fs_fclose( fs_fp );
}


static void fsts_put( char *source_name, char *target_name )
{
	FILE			*fi;
	fs_FILE			*fs_fp;
	unsigned char	bp[ RW_BUFFER_SIZE ];
	unsigned long	i;
	unsigned short	size;
	unsigned short	written_size;
	unsigned long	total	= 0;
	clock_t			t_meas;
	
	if ( NULL == (fi	= fopen( source_name, "rb" )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : local file open to read : \"%s\"\r\n", source_name );
		return;
	}

	if ( NULL == (fs_fp	= fs_fopen( target_name, W )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : file open @ fs_fopen( \"%s\", W )\r\n", target_name );
		fclose( fi );
		return;
	}
	
	mprintf( FSTS_NORM, CONTINUE, "\r\nputting file local:\"%s\" >> USB:\"%s\"\r\n", source_name, target_name );

	t_meas	= clock();
	
//	while ( size	= fread( bp, 1, RW_BUFFER_SIZE, fi ) )
	while ( 0 != (size	= fread( bp, 1, RW_BUFFER_SIZE, fi )) )
	{
		if ( size != (written_size	= fs_write( fs_fp, bp, size )) )		
		{
			mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nno space for more data. (%u)\r\n", written_size );
			debug_error++;
			break;
		}
		
		mprintf( FSTS_NORM, CONTINUE, "writing...  %ld bytes\r", total += size );
		wait_ms( 0 );
	}
	
	t_meas	= clock() - t_meas;
	
	if ( t_meas )
		mprintf( FSTS_NORM, CONTINUE, "written     %ld bytes     %.2f sec     %.2f B/sec\r", total += size, (double)t_meas / (double)CLK_TCK, (double)total / ((double)t_meas / (double)CLK_TCK) );

	fclose( fi );
	fs_fclose( fs_fp );
}


static void fsts_cp( char *source_name, char *target_name )
{
	fs_FILE			*fi;
	fs_FILE			*fo;
	unsigned char	bp[ RW_BUFFER_SIZE ];
	unsigned long	i;
	unsigned short	size;
	unsigned long	total	= 0;
	clock_t			t_meas;
	
	if ( NULL == (fi	= fs_fopen( source_name, R )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : file open @ fs_fopen( \"%s\", R )\r\n", source_name );
		return;
	}
	
	if ( NULL == (fo	= fs_fopen( target_name, W )) )
	{
		mprintf( FSTS_NORM, CONTINUE, "\r\nerror : file open @ fs_fopen( \"%s\", W )\r\n", target_name );
		fs_fclose( fi );
		return;
	}
	
	mprintf( FSTS_NORM, CONTINUE, "\r\ncopying file \"%s\" >> \"%s\"\r\n", source_name, target_name );

	t_meas	= clock();

//	while ( size	= fs_read( fi, bp, RW_BUFFER_SIZE ) )
	while ( 0 != (size	= fs_read( fi, bp, RW_BUFFER_SIZE )) )
	{
		if ( size != fs_write( fo, bp, size ) )		
		{
			mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nno space for more data\r\n" );
			break;
		}
		mprintf( FSTS_NORM, CONTINUE, "copying...  %ld bytes\r", total += size );
	}
	
	t_meas	= clock() - t_meas;
	
	if ( t_meas )
		mprintf( FSTS_NORM, CONTINUE, "copied     %ld bytes     %.2f sec     %.2f bytes/sec\r", total += size, (double)t_meas / (double)CLK_TCK, (double)total / ((double)t_meas / (double)CLK_TCK) );

	fs_fclose( fi );
	fs_fclose( fo );
}


static void fsts_cs( char *str )
{
	unsigned long	s;
	unsigned short	v;
	
	sscanf( str, "%X", &v );
	s	= fs_cluster_to_sector( v );
	
	mprintf( FSTS_NORM, CONTINUE, "\r\ncluster : %s  ... sector : %lu (0x%08lX)\r\n", str, s, s );
}


static void fsts_fat( char *str )
{
	unsigned short	start;
	unsigned short	i;
	
	sscanf( str, "%X", &start );
	start	/= 8;
	
	mprintf( FSTS_LIGHTGRAY, CONTINUE, "\r\nFAT view from 0x%04X", start );
	for ( i = start; i < (start + 64); i++ )
	{
		if ( !(i % 8) )
			mprintf( FSTS_LIGHTGRAY, CONTINUE, "\r\n  0x%04X  -", i );
			
		mprintf( FSTS_WHITE, CONTINUE, " %04X", fs_read_FAT( i ) );
	}
	mprintf( FSTS_LIGHTGRAY, CONTINUE, "\r\n" );
}


static void fsts_format( void )
{
	device_instance 		*dvi_ptr;
	storage_instance		*si_ptr;
	
	dvi_ptr		= storage_device_ready();
	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);

	if ( si_ptr->volume_instance[ 0 ] )
		fs_volume_close( si_ptr->volume_instance[ 0 ] );
	
	si_ptr->volume_instance[ 0 ]	= NULL;
	
	mprintf( FSTS_LIGHTGRAY, CONTINUE, "\r\nformatting..." );

	fs_format();
	
	mprintf( LIGHTGRAY, CONTINUE, "done\r\n" );

	si_ptr->volume_instance[ 0 ]	= fs_volume_open();	
}


#define		hp( x )			mprintf( FSTS_YELLOW, CONTINUE, "  %-7s", x )
#define		np( x )			mprintf( FSTS_NORM, CONTINUE, ": %-28s", x )
#define		br				mprintf( FSTS_NORM, CONTINUE, "\r\n" )

static void show_fsts_help( void )
{
	mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nHelp!\r\n" );
	hp( "e" );		np( "exit form shell" );			hp( "?" );		np( "help view" );				br;
	hp( "h" );		np( "view command history" );		hp( "i" );		np( "volume information" );		br;
	hp( "ls" );		np( "list of directory" );			hp( "cd" );		np( "change directory" );		br;
	hp( "pwd" );	np( "print working directory" );	hp( "mkdir" );	np( "make directory" );			br;
	hp( "mv" );		np( "move directory/file" );		hp( "rm" );		np( "remove directory/file" );	br;
	hp( "df" );		np( "free disk space" );			hp( "cp" );		np( "copy file" );				br;
	hp( "put" );	np( "file beam up (to USB)" );		hp( "get" );	np( "file beam dwon (fm USB)" );br;
	hp( "play" );	np( "WAV play on USB spkr" );		hp( "format" );	np( "format disk" );			br;
	hp( "dump" );	np( "dump sector contents" );		hp( "cs" );		np( "convert cluster->sector" );br;
	hp( "up/down arrow keys" );		np( "browse history" );
}




//////////
//////////
//////////	Audio play testing
//////////
//////////
#include		"class_dr/audio/sing.h"

fs_FILE			*g_play_source_file;
unsigned long	g_fsts_play_total;

static unsigned short 	storage_play_fill( short *buffer_ptr )
{
	unsigned short	read_size;

	read_size	= fs_read( g_play_source_file, (unsigned char *)buffer_ptr, (MEMORY_FILLING_SIZE_FROM_FILE << 1) );

	return ( read_size >> 1 );	//	size convert from bytes to short
}


static void storage_play_stop( void )
{
	if ( g_play_source_file )
		fs_fclose( g_play_source_file );
	
	g_play_source_file	= NULL;
		
	mprintf( FSTS_NORM, CONTINUE, "    strage play end. audio file closed.\r\n" );
}


void fsts_play( char *path )
{
	device_instance		*dvi_ptr;

	if ( is_audio_active() )
		return;
		
	if ( NULL == (g_play_source_file	= fs_fopen( path, R )) )
	{
		mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nerror : file open @ fs_fopen( \"%s\", R )\r\n", path );
		return;
	}
	
	if ( NULL == (dvi_ptr	= devep_find_class_interface( AUDIO_CLASS_INTERFACE, 0 )) )
	{
		mprintf( FSTS_LIGHTRED, CONTINUE, "\r\nerror : audio device is not ready.\r\n", path );
		
		fs_fclose( g_play_source_file );
		g_play_source_file	= NULL;
		
		return ;
	}
	
	if ( 0 != audio_initialize( dvi_ptr, "*", storage_play_fill, storage_play_stop ) )
		return;

	audio_start();
}


void fsts_stop( void )
{
//	if ( !is_audio_active() )
//		return;

//	audio_start();
}












