/*
**  WASABI-Hot! version 1.2b   
**
**
**      -- copyright (c) 2001-2003 by Philips Japan, Ltd. -- All rights reserved --
**
**
**      ** This code has been made to check/learn                          ** 
**      **                             the ISP1362/ISP1363 functionalities **
**      ** Release 08-Dec-2003                                             **
**
**      OKANO, Akifumi
**      
**		Application Laboratory, Portable Systems
**      Semiconductors Div, Philips Japan Ltd.
**      akifumi.okano@philips.com
**      +81-3-3740-4668
*/


/****************************************************************************/
/*	includes																*/
/****************************************************************************/

#include		<alloc.h>

#include		"_hc_core/atl_mix.h"
#include		"_hc_core/chap9.h"
#include		"_hc_core/dev_ep.h"
#include		"_hc_core/transfer.h"

#include		"_hc_cls/cls_hndl.h"

#include		"class_dr/storage/storage.h"
#include		"class_dr/storage/fs.h"

#include		"ui.h"
#include		"general.h"


/****************************************************************************/
/*	constants																*/
/****************************************************************************/

//#define		NO_DRIVE						0
#define		IDLE							1
#define		WAIT_FOR_CARD					2
#define		MOUNT_PROCEED					3
#define		MOUNTED							4
#define		UNMOUNT_PROCEED					5
#define		WAIT_STABILIZED					7
#define		WAIT_FOR_CARD_START				8

#define		STORAGE_CARD_WAIT_POLLING_INTERVAL			1000
#define		STORAGE_MOUNT_MONITOR_POLLING_INTERVAL		1000


/****************************************************************************/
/*	global vars																*/
/****************************************************************************/

device_instance 	*g_storage_device_ptr		= NULL;
unsigned long		g_payload_size;

unsigned long		g_cc_logger[ 16 ]	= { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

/****************************************************************************/
/*	function prototypes														*/
/****************************************************************************/


static void					strg_maintenance_routine_entry( void );
static void					strg_maintenance_routine( device_instance *dvi_ptr );
static unsigned char		check_the_card_exist( void );

static unsigned char		strg_command( unsigned char command_index, unsigned char *buffer_ptr, unsigned long logical_block_address, unsigned short allocation_length );
static unsigned char		bulk_only_transfer( st_command_block_wrapper *cbw_ptr, unsigned char *buffer_ptr, unsigned short allocation_length, unsigned char direction );
static void					un_stall( unsigned char direction );
static void					reset_recovery( void );
static unsigned char		ctrl_tr(  unsigned char *buffer_ptr, device_instance *dvi_ptr, unsigned short req, unsigned short length );

static unsigned char		strg_bulk_transfer( device_instance *dvi_ptr, unsigned char *buffer_ptr, unsigned short length, unsigned char direction );
static unsigned char		strg_bulk_transfer_once( device_instance *dvi_ptr, unsigned char *buffer_ptr, unsigned short length, unsigned char direction );
static transfer_instance	*start_strg_bulk_transfer( device_instance *dvi_ptr );
static void					end_strg_bulk_transfer( endpoint_info_ptr eip );

static unsigned char		strg_bulk_transfer_pp_once( device_instance *dvi_ptr, unsigned char *buffer_ptr, unsigned short length, unsigned char direction );
static						transfer_instance *start_strg_bulk_transfer_pp( device_instance *dvi_ptr, unsigned char buffer_switch );
static void					end_strg_bulk_transfer_pp( endpoint_info_ptr eip );


static void					storage_data_dump( char *s, unsigned char *b, unsigned short n );

void						strg_status_monitor( unsigned char dummy_for_compatibility );


/****************************************************************************/
/*	function definitions													*/
/****************************************************************************/

/*
==============================================================================
==============================================================================
====	Public functions for storage drive
====		Functions for APPLICATION side
==============================================================================
==============================================================================
*/


device_instance *storage_device_ready( void )
{
	return ( g_storage_device_ptr );
}


volume_param *storage_volume_ready( void )
{
	if ( storage_device_ready() )
		return ( (((storage_instance *)(g_storage_device_ptr->class_instance_ptr))->volume_instance)[ 0 ] );
	else
		return ( NULL );
}


unsigned char storage_device_address( void )
{
	if ( NULL == g_storage_device_ptr )
		return ( 0xFF );
	
	return ( g_storage_device_ptr->address );
}


unsigned char storage_mount( void )
{
	device_instance		*dvi_ptr;
	storage_instance	*si_ptr;

	if ( NULL == (dvi_ptr	= g_storage_device_ptr) )
		return ( error );
		
	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);

	if (si_ptr->state == WAIT_FOR_CARD)
		return ( no_error );
	if (si_ptr->state != IDLE)
		return ( error );

	si_ptr->state	= WAIT_FOR_CARD;

	return ( no_error );
}


unsigned char storage_unmount( void )
{
	unsigned char	err;
	
	err		= storage_remount( True );
	
	return ( err );
}


unsigned char storage_remount( unsigned char remount )
{
	device_instance		*dvi_ptr;
	storage_instance	*si_ptr;
	
	if ( NULL == (dvi_ptr	= g_storage_device_ptr) )
		return ( error );
		
	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);

	if (si_ptr->state != MOUNTED)
		return ( error );

	si_ptr->state	= UNMOUNT_PROCEED;
	si_ptr->remount	= remount;
	
	return ( no_error );
}


unsigned short storage_read_sector( unsigned char *buffer_ptr, unsigned long logical_block_address, unsigned short num_of_sector )
{
	if ( strg_command( ST_READ, buffer_ptr, logical_block_address, num_of_sector ) )
		return ( 0 );
	else
		return ( num_of_sector );
}


unsigned short storage_write_sector( unsigned char *buffer_ptr, unsigned long logical_block_address, unsigned short num_of_sector )
{
	if ( strg_command( ST_WRITE, buffer_ptr, logical_block_address, num_of_sector ) )
		return ( 0 );
	else
		return ( num_of_sector );
}


extern volume_param	g_volume_parameter;

void storage_info( void )
{
	volume_param	*vpp;
	vpp		= &g_volume_parameter;
	
	mprintf( LIGHTGRAY, CONTINUE, "\r\n  Volume parameters (file system = %s)\r\n",	(vpp->file_system) ? "FAT12" : "FAT16" );

	mprintf( LIGHTGRAY, CONTINUE, "    bytes/sector      = %8u bytes",		vpp->bytes_per_sector );
		mprintf( LIGHTGRAY, CONTINUE, "    reserved sectors        = %8u\r\n",		vpp->reserved_sectors );
	mprintf( LIGHTGRAY, CONTINUE, "    sector/cluster    = %8u sectors",		vpp->sector_per_cluster );
		mprintf( LIGHTGRAY, CONTINUE, "  number of FATs          = %8u\r\n",		vpp->number_of_FATs );
	mprintf( LIGHTGRAY, CONTINUE, "    bytes/cluster     = %8u bytes",	vpp->bytes_per_cluster );
		mprintf( LIGHTGRAY, CONTINUE, "    root directory entries  = %8u\r\n",	vpp->root_directory_entries );
	mprintf( LIGHTGRAY, CONTINUE, "    total sectors     = %8lu sectors",		vpp->total_sectors );
		mprintf( LIGHTGRAY, CONTINUE, "  number of FAT sectors   = %8u\r\n",		vpp->number_of_FAT_sectors );


	mprintf( LIGHTGRAY, CONTINUE, "    start_sector_FAT1                = 0x%04X\r\n", vpp->start_sector_FAT1 );
	mprintf( LIGHTGRAY, CONTINUE, "    start_sector_FAT2                = 0x%04X\r\n", vpp->start_sector_FAT2 );
	mprintf( LIGHTGRAY, CONTINUE, "    start_sector_root_directory      = 0x%04X\r\n", vpp->start_sector_root_directory );
	mprintf( LIGHTGRAY, CONTINUE, "    start_sector_claster_region      = 0x%04X\r\n", vpp->start_sector_claster_region );
}


#include		<stdio.h>
#define			DUMP_INFO_DATA_SIZE		512

void storage_dump_info( unsigned long start_sec, unsigned long length )
{
	FILE			*fp;
	unsigned long	i;
	unsigned short	j;
	volume_param	*vpp;
	
	mprintf( LIGHTGRAY, CONTINUE, "\r\nvolume dump..." );
	
	if ( NULL == (fp	= fopen( "storage.txt", "w" )) )
	{
		mprintf( LIGHTGRAY, CONTINUE, "file open error\r\n" );
		return;
	}
	
	vpp		= &g_volume_parameter;

	fprintf( fp, "bytes_per_sector        = 0x%04X\n",		vpp->bytes_per_sector );
	fprintf( fp, "sector_per_cluster      = 0x%02X\n",		vpp->sector_per_cluster );
	fprintf( fp, "reserved_sectors        = 0x%02X\n",		vpp->reserved_sectors );
	fprintf( fp, "number_of_FATs          = 0x%02X\n",		vpp->number_of_FATs );
	fprintf( fp, "root_directory_entries  = 0x%04X (%d entries)\n",	    vpp->root_directory_entries, vpp->root_directory_entries );
	fprintf( fp, "number_of_FAT_sectors   = 0x%04X\n",		vpp->number_of_FAT_sectors );

	
	fprintf( fp, "start_sector_FAT1                = 0x%04X\n", vpp->start_sector_FAT1 );
	fprintf( fp, "start_sector_FAT2                = 0x%04X\n", vpp->start_sector_FAT2 );
	fprintf( fp, "start_sector_root_directory      = 0x%04X\n", vpp->start_sector_root_directory );
	fprintf( fp, "start_sector_claster_region      = 0x%04X\n\n\n", vpp->start_sector_claster_region );
				
	for ( i = start_sec; i < start_sec + length; i++ )
	{
		unsigned char	data[ DUMP_INFO_DATA_SIZE ];
		strg_command( ST_READ, data, i, 1 );
		
		{
			
			fprintf(fp, "dump:lba=%lu (0x%04lX)", i, i );

			for ( j = 0; j < DUMP_INFO_DATA_SIZE; j++ )
			{
				if ( !(j % 16) )
					fprintf( fp, "\n%04d(0x%04X) : ", j, j );
			
				fprintf( fp, " %02X", *(data + j) );
			}

			fprintf( fp, "\n" );
		}
	}
	
	fclose( fp );
	
	mprintf( LIGHTGRAY, CONTINUE, "done\r\n" );
}




/*
==============================================================================
==============================================================================
====	Public functions for storage drive
====		Functions for HOST CONTROLLER side
==============================================================================
==============================================================================
*/


/*********															*********/
/*																			*/
/*	Device class handling functions											*/
/*																			*/
/*		This functions is called from functions in cls_hndl.c as callback	*/
/*		functions. 															*/
/*		All these functions are needed to be installed by 					*/
/*		"clshndl_initialization_method_install()"	in "cls_hndl.c".			*/
/*																			*/
/*		These functions will be called at storage device attach/detach		*/
/*		events. 															*/
/*																			*/
/*********															*********/


unsigned short storage_init_commands( device_instance *dvi_ptr )
{
	storage_instance		*si_ptr;
	endpoint_info_ptr		eip;
	unsigned char			buffer;
	unsigned short			err;
	unsigned char			i;
	
	if ( devep_regist_config_descriptors( dvi_ptr, 1, 0 ) )
	{
		mprintf( RED, CONTINUE, "error : storage_init_commands\r\n" );
		return ( error );	//	to dispose device
	}
	

	if ( g_storage_device_ptr )
	{
		mprintf( RED, CONTINUE, "error : No multiple storage device supprt\r\n" );
		return ( error );	//	to dispose device
	}
	
	//	Check descriptor
	
	if ( 0x06 != (dvi_ptr->interfacef_descriptor).bInterfaceSubClass )
	{
		mprintf( RED, CONTINUE, "error : This is not compatible to SCSI command set. bInterfaceSubClass=0x%02X\r\n", (dvi_ptr->interfacef_descriptor).bInterfaceSubClass );
		return ( error );	//	to dispose device
	}
	
	if ( 0x50 != (dvi_ptr->interfacef_descriptor).bInterfaceProtocol )
	{
		mprintf( RED, CONTINUE, "error : This does not support bulk-only transport. bInterfaceProtocol=0x%02X\r\n", (dvi_ptr->interfacef_descriptor).bInterfaceSubClass );
		return ( error );	//	to dispose device
	}
	
	/*
	**	Make a storage instance for storage device
	*/

	if ( NULL == (si_ptr		= (storage_instance *)malloc( sizeof( storage_instance ) )) )
		mprintf( LIGHTRED, CONTINUE, "error @ storage_init_commands : making storage class instance.\r\n" );
		
	dvi_ptr->class_instance_ptr		= si_ptr;



	/*********
	 Is this RESET command needed?
	 *********/	
	ctrl_tr( &buffer, dvi_ptr, storage_cls_sp_request_BulkOnlyMassStorageReset, 0 );
	
	err		= ctrl_tr( &buffer, dvi_ptr, storage_cls_sp_request_GetMaxLUN, 1 );

	if ( (err & 0x00FF) == 0x4 )
		si_ptr->max_LUN		= 0;
	else
		si_ptr->max_LUN		= buffer;
	
	si_ptr->target_LUN	= 0;

	//	Find first 2 (IN and OUT) endpoints for main communication
	
	for ( i = 0; i < MAX_ENDPOINT_NUM; i++ )
	{
		if ( NULL == ( eip		= (dvi_ptr->epi_ptr)[ EpOUT ][ i ] ) )
			continue;
	
		if ( eip->transfer_type == Ep_TransferType_Bulk )
		{
			si_ptr->endpoint_OUT	= eip;
			si_ptr->ep_out			= i;
			break;
		}
	}
	
	for ( i = 0; i < MAX_ENDPOINT_NUM; i++ )
	{
		if ( NULL == ( eip		= (dvi_ptr->epi_ptr)[ EpIN  ][ i ] ) )
			continue;
	
		if ( eip->transfer_type == Ep_TransferType_Bulk )
		{
			si_ptr->endpoint_IN		= eip;
			si_ptr->ep_in			= i | 0x80;
			break;
		}
	}
	
	si_ptr->volume_instance[ 0 ]	= NULL;
	si_ptr->state					= IDLE;
	si_ptr->remount					= True;
	
	g_payload_size					= atlmix_atl_payload_size();

	g_storage_device_ptr			= dvi_ptr;

	gene_install_asynchronous_periodic_process( 8, strg_maintenance_routine_entry );

	storage_mount();
	
	si_ptr->previous_state	= 0xFF;
	ui_install_status_monitor_custom_routine( STORAGE_STATUS_MONITOR_LINE, strg_status_monitor );

//	mprintf( LIGHTGRAY, CONTINUE, "  storage class : class driver started, Bulk-only transfer/SCSI protocol\r\n" );

	return ( no_error );
}


unsigned short storage_dispose_process( device_instance *dvi_ptr )
{
	storage_instance		*si_ptr;

	ui_install_status_monitor_custom_routine( STORAGE_STATUS_MONITOR_LINE, NULL );
	gene_install_asynchronous_periodic_process( 8, NULL );

	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);
	
	if ( si_ptr->volume_instance[ 0 ] != NULL )
	{
		mprintf( RED, CONTINUE, "error : volume removed before volume-close\r\n" );
		fs_volume_close( si_ptr->volume_instance[ 0 ] );
		
		si_ptr->volume_instance[ 0 ]	= NULL;
	}
	
	free( si_ptr );
	dvi_ptr->class_instance_ptr		= NULL;
	
	g_storage_device_ptr			= NULL;
	
	return ( no_error );
}



/*
==============================================================================
==============================================================================
====	Private functions for storage drive
====		Storage device maintenance
==============================================================================
==============================================================================
*/


static void strg_maintenance_routine_entry( void )
{
	static unsigned short	is_in_execution		= False;

	if ( is_in_execution )
		return;
	else
		is_in_execution		= True;


	if ( !g_storage_device_ptr )
		return;
	
	strg_maintenance_routine( g_storage_device_ptr );
	
	if ( is_in_execution )
		is_in_execution		= False;
}


static void strg_maintenance_routine( device_instance *dvi_ptr )
{
	static unsigned long			poll_timing		= 0;
	static unsigned char			wait_stable;
	static unsigned long			wait_stable_count;
	unsigned char					data[ 36 ];
	unsigned char					err;
	unsigned char					check;
	char							k;
	storage_instance				*si_ptr;
static unsigned long			sss		= 0;
	
	if ( NULL == dvi_ptr )
		return;
	
	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);

	switch ( si_ptr->state )
	{
		case IDLE :
			break;
			
		case WAIT_FOR_CARD_START :
		
			poll_timing		= 0;
			si_ptr->state	= WAIT_FOR_CARD;
			//break;
		
		case WAIT_FOR_CARD :
		
			if ( poll_timing < gp_sof_counter )
			{
				poll_timing		= gp_sof_counter + STORAGE_CARD_WAIT_POLLING_INTERVAL;

				for ( (si_ptr->target_LUN) = 0; (si_ptr->target_LUN) <= (si_ptr->max_LUN) ; )
				{
					check 	= check_the_card_exist();
					
					if ( (check == 0) || (check == 6) )
					{
						wait_stable			= 0xFF;
						wait_stable_count	= 0;
						si_ptr->state		= WAIT_STABILIZED;
						return;		//	return and wait for retry
					}
					(si_ptr->target_LUN)++;
				}
			}
			else if ( (poll_timing - gp_sof_counter) > 5000 )
			{
				si_ptr->state		= WAIT_FOR_CARD_START;
			}

			break;

		case WAIT_STABILIZED :
		
			if ( poll_timing < gp_sof_counter )
			{
				poll_timing		= gp_sof_counter + 10;
				
				if (wait_stable_count++ > 500)
				{
//					si_ptr->state	= IDLE;
					si_ptr->state	= WAIT_FOR_CARD_START;
					return;
				}
				
				wait_stable		= (wait_stable << 1) | ((strg_command( TEST_UNIT_READY, NULL, 0, 0 ) == 0) ? 0x0 : 0x1);
				
				if ( !wait_stable )
				{
					si_ptr->state	= MOUNT_PROCEED;
					return;		//	return and wait for retry
				}
			}

			break;
			
		case MOUNT_PROCEED :
		
			mprintf( WHITE, CONTINUE, "\r\n  storage class : Card ready.    " );
			
			strg_command( REQUEST_SENSE, data, 0, 18 );
			
			if ( data[ 2 ] && (data[ 2 ] != 0x5) )
			{
				mprintf( WHITE,     CONTINUE, "ATTENTION!!!!!!!!!   0x%X  %lu\r\n", data[ 2 ], gp_drive_parameter.block_length );
				mprintf( LIGHTRED, CONTINUE, "Non recoverable error in this version. Please re-attach USB device.\r\n" );
				
				si_ptr->state	= IDLE;
				return;
			}

			strg_command( INQUIRY,                      data, 0, 36 );
			strg_command( PREVENT_ALLOW_MEDIUM_REMOVAL, NULL, 0,  1 );
			strg_command( READ_CAPACITY,                data, 0,  0 );
			
			gp_drive_parameter.last_LBA			= long_swap( *(((unsigned long *)data) + 0) );
			gp_drive_parameter.block_length		= long_swap( *(((unsigned long *)data) + 1) );
			
			mprintf( LIGHTGRAY, CONTINUE, "size = %lu bytes ", gp_drive_parameter.last_LBA * gp_drive_parameter.block_length );
			mprintf( LIGHTGRAY, CONTINUE, "(%ld MB)\r\n", (gp_drive_parameter.last_LBA * gp_drive_parameter.block_length) >> 20 );
			
			if ( NULL == (si_ptr->volume_instance[ 0 ]		= fs_volume_open()) )
			{
				mprintf( WHITE, CONTINUE, "  storage class : Volume mount failed\r\n" );
				
				mprintf( WHITE, CONTINUE, "            Format? (y|n)  " );
				

				while ( 0 == (k	= ui_key_wait() & 0xFF) )
				{
					mprintf( WHITE, CONTINUE, "\r                               \r\n" );

					if ( !storage_volume_ready() )
						return;
						
					if ( si_ptr->state != MOUNT_PROCEED )
						return;
				}
					
				if ( 'y' == k )
				{
					mprintf( WHITE, CONTINUE, "\r\n  storage class : Format started..." );

					if ( si_ptr->volume_instance[ 0 ] != NULL )
						fs_volume_close( si_ptr->volume_instance[ 0 ] );			

					si_ptr->volume_instance[ 0 ]	= NULL;
					
					fs_format();
					si_ptr->state	= IDLE;
					return;
				}
				si_ptr->state	= IDLE;
				break;
			}
			
			mprintf( WHITE, CONTINUE, "  storage class : Volume mounted." );
			mprintf( LIGHTGRAY, CONTINUE, "   ...press" );
			mprintf( LIGHTRED, CONTINUE, " [s] " );
			mprintf( LIGHTGRAY, CONTINUE, "to go into test shell.\r\n" );
			
			si_ptr->state	= MOUNTED;
			
			break;
			
		case MOUNTED :
		
			if ( gp_sof_counter < poll_timing )
				return;
				
			poll_timing		= gp_sof_counter + STORAGE_MOUNT_MONITOR_POLLING_INTERVAL;
			
			if ( 0 == check_the_card_exist() )	//	0 : the card exists
				return;

			if ( si_ptr->volume_instance[ 0 ] != NULL )
				mprintf( RED, CONTINUE, "\r\n  storage class : error : volume removed before volume-close\r\n" );

			//	break;	//	process continued to next state...
			
		case UNMOUNT_PROCEED :
		
			if ( NULL != si_ptr->volume_instance[ 0 ] )
				fs_volume_close( si_ptr->volume_instance[ 0 ] );
				
			si_ptr->volume_instance[ 0 ]		= NULL;
			
			strg_command( PREVENT_ALLOW_MEDIUM_REMOVAL, NULL, 0, 0 );
			strg_command( START_STOP_UNIT,              NULL, 0, 2 );	//	UnLoad media

			mprintf( WHITE, CONTINUE, "  storage class : Volume unmounted\r\n" );

			if ( si_ptr->remount == False )
				si_ptr->state	= IDLE;
			else
				si_ptr->state	= WAIT_FOR_CARD;

			break;

	}
}


static unsigned char check_the_card_exist( void )
{
	unsigned char	data[ 18 ];

	if ( strg_command( TEST_UNIT_READY, NULL, 0, 0 ) )
	{
		strg_command( REQUEST_SENSE, data, 0, 18 );

		return ( data[ 2 ] );
	}

	return ( 0 );
}




/*
==============================================================================
==============================================================================
====	Private functions for storage drive
====		Data transfers (command level)
==============================================================================
==============================================================================
*/


static unsigned char strg_command( unsigned char command_index, unsigned char *buffer_ptr, unsigned long logical_block_address, unsigned short allocation_length )
{
#define		COMMAND_OUT		0x00
#define		COMMAND_IN		0x80

	static unsigned char		commands[ 9 ][ 2 ]		=	{
																{ 0x06, 0x03 },		//	REQUEST_SENSE
																{ 0x06, 0x12 },		//	INQUIRY	
																{ 0x06, 0x1A },		//	MODE_SENSE
																{ 0x0A, 0x25 },		//	READ_CAPACITY
																{ 0x0A, 0x28 },		//	ST_READ
																
																{ 0x06, 0x00 },		//	TEST_UNIT_READY
																{ 0x06, 0x1B },		//	START_STOP_UNIT
																{ 0x06, 0x1E },		//	PREVENT_ALLOW_MEDIUM_REMOVAL
																{ 0x0A, 0x2A } 		//	ST_WRITE
															};
	st_command_block_wrapper	cbw;
	st_command_status_wrapper	csw;
	unsigned char				comm_direction;
	unsigned char				CBWStatus_value;
	unsigned char				i;

	if ( commands[ command_index ][ 0 ] == 0x06 )
	{
		cbw.CBWCB.reserved					= 0;
		cbw.CBWCB.transfer_length			= 0;
		cbw.CBWCB.control					= 0;

		((st_command_block6 *)(&(cbw.CBWCB)))->logical_block_address	= long_swap( logical_block_address );
		((st_command_block6 *)(&(cbw.CBWCB)))->transfer_length			= (unsigned char)allocation_length;
		((st_command_block6 *)(&(cbw.CBWCB)))->control					= 0;
		*((unsigned char *)(&(cbw.CBWCB)))								= commands[ command_index ][ 1 ];

		for ( i = 0; i < 10; i++ )
			(((st_command_block6 *)(&(cbw.CBWCB)))->dummy)[ i ]			= 0;
		
	}
	else
	{
		cbw.CBWCB.opcode					= commands[ command_index ][ 1 ];
		cbw.CBWCB.service_action			= 0;
		cbw.CBWCB.logical_block_address		= long_swap( logical_block_address );
		cbw.CBWCB.reserved					= 0;

		if ( READ_CAPACITY == command_index )
			cbw.CBWCB.transfer_length		= 0;
		else
			cbw.CBWCB.transfer_length		= short_swap( allocation_length );
			
		cbw.CBWCB.control					= 0;
		
		for ( i = 0; i < 6; i++ )
			(cbw.CBWCB.dummy)[ i ]			= 0;
	}
	
	comm_direction		= (command_index < 5) ? COMMAND_IN : COMMAND_OUT;

	if ( PREVENT_ALLOW_MEDIUM_REMOVAL == command_index )
		allocation_length	= 0;
	else if ( START_STOP_UNIT == command_index )
		allocation_length	= 0;
	else if ( READ_CAPACITY == command_index )
		allocation_length	= 8;
	else if ( (ST_READ == command_index) || (ST_WRITE == command_index) )
		allocation_length	*= gp_drive_parameter.block_length;
	
	cbw.dCBWSignature				= 0x43425355;
//	cbw.dCBWTag						= command_tag;
	cbw.dCBWDataTransferLength		= allocation_length;
	cbw.bmCBWFlags					= comm_direction;
//	cbw.bmCBWLUN					= 0;
	cbw.bmCBWLength					= commands[ command_index ][ 0 ];

	CBWStatus_value		= bulk_only_transfer( &cbw, buffer_ptr, allocation_length, comm_direction );

	if ( CBWStatus_value == 2 )
		reset_recovery();
		
	return ( CBWStatus_value );
}


static unsigned char bulk_only_transfer( st_command_block_wrapper *cbw_ptr, unsigned char *buffer_ptr, unsigned short allocation_length, unsigned char direction )
{
	static unsigned long		command_tag		= 0;
	st_command_status_wrapper	csw;
	unsigned char				err;
	unsigned char				retry			= 0;
	unsigned char				return_code		= 0;

	command_tag++;
	
	cbw_ptr->dCBWTag	= command_tag;
	cbw_ptr->bmCBWLUN	= ((storage_instance *)(g_storage_device_ptr->class_instance_ptr))->target_LUN;

	//
	//	CBW
	//

	//if ( err	= storage_bulk_transfer( g_storage_device_ptr, (unsigned char *)cbw_ptr, sizeof( st_command_block_wrapper ), OUT ) )
	if ( 0 != (err	= strg_bulk_transfer( g_storage_device_ptr, (unsigned char *)cbw_ptr, sizeof( st_command_block_wrapper ), OUT )) )
	{	
		if ( err == 0x4 )
			un_stall( OUT );
		else
		{
			allocation_length	= 0;
			retry				= 2;
		}
	}

	//
	//	DATA
	//
	if ( allocation_length )
	{
		//if ( err	= storage_bulk_transfer( g_storage_device_ptr, buffer_ptr, allocation_length, (direction == COMMAND_OUT) ? OUT : IN ) )
		if ( 0 != (err	= strg_bulk_transfer( g_storage_device_ptr, buffer_ptr, allocation_length, (direction == COMMAND_OUT) ? OUT : IN )) )
		{
			if ( err == 0x4 )
				un_stall( (direction == COMMAND_OUT) ? OUT : IN );
			else
				retry	= 2;
		}
	}

	//
	//	CSW
	//
	while ( retry < 2 )
	{
		err		= strg_bulk_transfer( g_storage_device_ptr, (unsigned char *)(&csw), sizeof( st_command_status_wrapper ), IN );

		if ( err != 0x4 )
			break;
		else
		{
			if ( retry == 0 )
				un_stall( IN );
		}
		
		retry++;
	}
	
	if ( retry == 2 )
	{
		return_code		= 2;
	}
	else
	{
		if ( (err == 0) || (err == 4) )
			return_code		= csw.bmCBWStatus;
		else
			return_code		= 3;
	}


	if ( return_code == 3 )
	{
		if ( ((storage_instance *)(g_storage_device_ptr->class_instance_ptr))->volume_instance[ 0 ] != NULL )
		{
			mprintf( RED, CONTINUE, "error : un-recoverable error happened  err=0x%01X\r\n", err );
			fs_volume_close( ((storage_instance *)(g_storage_device_ptr->class_instance_ptr))->volume_instance[ 0 ] );
			
			((storage_instance *)(g_storage_device_ptr->class_instance_ptr))->volume_instance[ 0 ]	= NULL;
		}
	}
	
	return ( return_code );
}


static void un_stall( unsigned char direction )
{
	device_instance 	*dvi_ptr;
	storage_instance	*si_ptr;
	unsigned long		dummy;
	unsigned short		size;
	
	dvi_ptr		= g_storage_device_ptr;
	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);
	
	size	= 0;
	devep_std_request( (unsigned char *)(&dummy), dvi_ptr, std_Dv_request_CLEAR_FEATURE_Ep, 0, (direction == OUT) ? si_ptr->ep_out : si_ptr->ep_in, &size );
}


static void reset_recovery( void )
{
	device_instance		*dvi_ptr;
	storage_instance	*si_ptr;
	unsigned char		buffer;

	dvi_ptr		= g_storage_device_ptr;

	ctrl_tr( &buffer, dvi_ptr, storage_cls_sp_request_BulkOnlyMassStorageReset, 0 );

	un_stall( IN  );
	un_stall( OUT );
}

static unsigned char ctrl_tr(  unsigned char *buffer_ptr, device_instance *dvi_ptr, unsigned short req, unsigned short length )
{
	unsigned short	err;
	unsigned short	size;
	unsigned char	i		= 0;
	
	while ( i++ < 3 )
	{
		size	= length;

		err		 = devep_std_request( buffer_ptr, dvi_ptr, req, 0, 0, &size );
		err		&= 0xF;
		
		if ( (err == 0) || (err == 4) )
			break;
	}
	
	return ( err );
}



/*
==============================================================================
==============================================================================
====	Private functions for storage drive
====		Data transfers (low level : Bulk transfers)
==============================================================================
==============================================================================
*/


static unsigned char strg_bulk_transfer( device_instance *dvi_ptr, unsigned char *buffer_ptr, unsigned short length, unsigned char direction )
{
	unsigned char	result;
	unsigned char	i	= 0;

	do
	{
	
#ifdef USE_PAIRED_PTD

		atlmix_pp_atl_flag_mask( True );

		if ( (length > g_payload_size) && !(atlmix_pp_atl_active()) && !i )
			result	= strg_bulk_transfer_pp_once( dvi_ptr, buffer_ptr, length, direction );
		else
			result	= strg_bulk_transfer_once( dvi_ptr, buffer_ptr, length, direction );
		
		atlmix_pp_atl_flag_mask( False );
		
#else	//	USE_PAIRED_PTD
			
		result	= strg_bulk_transfer_once( dvi_ptr, buffer_ptr, length, direction );

#endif	//	USE_PAIRED_PTD

		g_cc_logger[result]++;

		if ( (result == 0xF) || (result == 0x4) )
			break;

//		mprintf( LIGHTRED, CONTINUE, "result=0x%X (%u)\r\n", result, i );
	}
	while ( i++ < 3 );
	
	return ( (result == 0xF) ? 0 : result );
}


void storage_show_cclog_result( void )
{
	unsigned char	i;
	
	mprintf( LIGHTRED, CONTINUE, "\r\n" );

	for ( i = 0; i < 16; i++ )
	{
		if ( !(i % 4) )
			mprintf( LIGHTRED, CONTINUE, "  | " );
	
		mprintf( LIGHTRED, CONTINUE, " %lu", g_cc_logger[i] );
	}
	
	
	mprintf( LIGHTRED, CONTINUE, "\r\n" );
}


static unsigned char strg_bulk_transfer_once( device_instance *dvi_ptr, unsigned char *buffer_ptr, unsigned short length, unsigned char direction )
{
	storage_instance	*si_ptr;
	unsigned long		timeout;

	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);

	si_ptr->ti_index			= atlmix_get_open_transfer_index( BULK_TRANSFER );	

	if ( NO_OPEN_ATL_TRANSFER_SLOT == si_ptr->ti_index )
	{
		atlmix_display_open_slot();
		mprintf( LIGHTRED, ABORT, "error @ strg_bulk_transfer_once, transfer list full.\r\n" );
	}
		
	si_ptr->remaining_tr_size	= length;
	si_ptr->transferred_size	= 0;
	si_ptr->transfer_stop		= False;
	si_ptr->bulk_done			= 0;
	si_ptr->buffer_ptr			= buffer_ptr;
	si_ptr->direction			= direction;

	si_ptr->tip		= start_strg_bulk_transfer( dvi_ptr );

	if ( NULL == si_ptr->tip )
	{
		/*	Error at starting transfer	*/;

		atlmix_free_transfer_index( BULK_TRANSFER, si_ptr->ti_index );	
		
		return ( error );
	}
	
	timeout			= gp_sof_counter + 500;
	
	while ( !si_ptr->bulk_done && (gp_sof_counter < timeout) )
		;
	
	si_ptr->transfer_stop		= True;

	if ( si_ptr->remaining_tr_size && !si_ptr->bulk_done )
		si_ptr->bulk_done	= 0xE;

	atlmix_free_transfer( si_ptr->tip );		/*	release the slot	*/
	si_ptr->tip		= NULL;

	return ( si_ptr->bulk_done );
}


static transfer_instance *start_strg_bulk_transfer( device_instance *dvi_ptr )
{
	transfer_instance	*transfer_inst_pointer;
	storage_instance	*si_ptr;

	si_ptr					= (storage_instance *)(dvi_ptr->class_instance_ptr);

	transfer_inst_pointer	= atlmix_set_transfer(
									BULK_TRANSFER,
									0,
									si_ptr->ti_index,
									si_ptr->buffer_ptr,
									si_ptr->remaining_tr_size,
									si_ptr->direction,
									(si_ptr->direction == OUT) ? si_ptr->endpoint_OUT : si_ptr->endpoint_IN,
									end_strg_bulk_transfer
							);

	return ( transfer_inst_pointer );	/*	Started!	*/
}


static void end_strg_bulk_transfer( endpoint_info_ptr eip )
{
	//	Callback function for the transfer (called in ISR context)								*/

	storage_instance	*si_ptr;
	transfer_instance	*tip;
	unsigned short		size;

	tip			= eip->transfer_instance;
	si_ptr		= (storage_instance *)(g_storage_device_ptr->class_instance_ptr);

	size		= tip->transferred_size;
	
	si_ptr->buffer_ptr			+= size;
	si_ptr->transferred_size	+= size;
	si_ptr->remaining_tr_size	-= size;
	
	if ( tip->completion_code )
	{
		si_ptr->bulk_done	= tip->completion_code;
		return;
	}
	

	if ( si_ptr->remaining_tr_size == 0 )
		si_ptr->bulk_done	= 0xF;	
	else if ( !si_ptr->transfer_stop )
		start_strg_bulk_transfer( g_storage_device_ptr );	//	Retry to transfer remaining data
}


//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

#ifdef		USE_PAIRED_PTD
#define		PP_START	0xFF

static unsigned char strg_bulk_transfer_pp_once( device_instance *dvi_ptr, unsigned char *buffer_ptr, unsigned short length, unsigned char direction )
{
	storage_instance	*si_ptr;
	unsigned long		timeout;

	si_ptr		= (storage_instance *)(dvi_ptr->class_instance_ptr);

	si_ptr->ti_index0			= atlmix_get_open_transfer_index( BULK_TRANSFER );	
	
	if ( NO_OPEN_ATL_TRANSFER_SLOT == si_ptr->ti_index0 )
		mprintf( LIGHTRED, ABORT, "error @ strg_bulk_transfer_pp_once, transfer list full.\r\n" );

	si_ptr->ti_index1			= atlmix_get_open_transfer_index( BULK_TRANSFER );	

	if ( NO_OPEN_ATL_TRANSFER_SLOT == si_ptr->ti_index1 )
	{
		atlmix_free_transfer_index( BULK_TRANSFER, si_ptr->ti_index0 );
		return ( strg_bulk_transfer_once( dvi_ptr, buffer_ptr, length, direction ) );
	}
	
	atlmix_pp_link_indexes( si_ptr->ti_index0, si_ptr->ti_index1 );

	si_ptr->remaining_tr_size	= length;
	si_ptr->transferred_size	= 0;
	si_ptr->transfer_stop		= False;
	si_ptr->bulk_done			= 0;
	si_ptr->buffer_ptr			= buffer_ptr;
	si_ptr->direction			= direction;
	
	si_ptr->tip0		= start_strg_bulk_transfer_pp( dvi_ptr, 0 | PP_START );
	si_ptr->tip1		= start_strg_bulk_transfer_pp( dvi_ptr, 1            );

	if ( (NULL == si_ptr->tip0) || (NULL == si_ptr->tip1) )
	{
		if ( NULL != si_ptr->tip0 )
			atlmix_free_transfer( si_ptr->tip0 );		/*	release the slot	*/

		if ( NULL != si_ptr->tip1 )
			atlmix_free_transfer( si_ptr->tip1 );		/*	release the slot	*/
		
		/*	Error at starting transfer	*/;
		return ( error );
	}
	
	atlmix_pp_start();

	timeout			= gp_sof_counter + 500;
	
	while ( !si_ptr->bulk_done && (gp_sof_counter < timeout) )
		;
	
	si_ptr->transfer_stop		= True;

	if ( si_ptr->remaining_tr_size && !si_ptr->bulk_done )
		si_ptr->bulk_done	= 0xE;

	atlmix_pp_unlink_indexes();

	atlmix_free_transfer( si_ptr->tip0 );		/*	release the slot	*/
	atlmix_free_transfer( si_ptr->tip1 );		/*	release the slot	*/
	si_ptr->tip0		= NULL;
	si_ptr->tip1		= NULL;

	return ( si_ptr->bulk_done );
}


static transfer_instance *start_strg_bulk_transfer_pp( device_instance *dvi_ptr, unsigned char buffer_switch )
{
	transfer_instance		*transfer_inst_pointer;
	storage_instance		*si_ptr;
	static unsigned short	written_size;

	si_ptr					= (storage_instance *)(dvi_ptr->class_instance_ptr);

	if ( buffer_switch == PP_START )
	{
		written_size	= 0;
		buffer_switch	= 0;
	}

	transfer_inst_pointer	= atlmix_set_transfer(
									BULK_TRANSFER,
									0,
									buffer_switch ? si_ptr->ti_index1 : si_ptr->ti_index0,
									si_ptr->buffer_ptr + written_size,
									g_payload_size,
									si_ptr->direction,
									(si_ptr->direction == OUT) ? si_ptr->endpoint_OUT : si_ptr->endpoint_IN,
									end_strg_bulk_transfer_pp
							);
	
	written_size	+= g_payload_size;

	return ( transfer_inst_pointer );	/*	Started!	*/
}


static void end_strg_bulk_transfer_pp( endpoint_info_ptr dummy )
{
	//	Callback function for the transfer (called in ISR context)								*/

	storage_instance	*si_ptr;
	transfer_instance	*tip;
	unsigned short		size;
	unsigned char		buffer_switch;
	
	dummy	= dummy;


	buffer_switch	= atlmix_pp_which_buffer();
	si_ptr			= (storage_instance *)(g_storage_device_ptr->class_instance_ptr);

	tip			= buffer_switch ? si_ptr->tip1 : si_ptr->tip0;
	size		= tip->transferred_size;
	
	si_ptr->transferred_size	+= size;
	si_ptr->remaining_tr_size	-= size;

	if ( tip->completion_code )		//	Error happened
	{
		si_ptr->bulk_done	= tip->completion_code;
		mprintf( WHITE, CONTINUE, "\r\nsize=%u, completion_code=%u\r\n", size, tip->completion_code );
		return;
	}

	if ( si_ptr->remaining_tr_size == 0 )
	{
		si_ptr->bulk_done	= 0xF;	
	}
	else if ( si_ptr->remaining_tr_size == g_payload_size )
	{
		//	do nothing and wait for the last transfer completion
	}
	else if ( si_ptr->remaining_tr_size < g_payload_size )
	{
		mprintf( LIGHTGRAY, CONTINUE, "???????????????????" );
	}
	else if ( !si_ptr->transfer_stop )
	{
		start_strg_bulk_transfer_pp( g_storage_device_ptr, buffer_switch );	//	Fill and start remaining data transfer
	}
}
#endif	//	USE_PAIRED_PTD


//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

static void storage_data_dump( char *s, unsigned char *b, unsigned short n )
{
	//	Debug purpose
	
	unsigned short	i;
	
	mprintf( LIGHTGRAY, CONTINUE, "dump:%s  (%p)", s, b );

	for ( i = 0; i < n; i++ )
	{
		if ( !(i % 16) )
			mprintf( LIGHTGRAY, CONTINUE, "\r\n" );
	
		mprintf( LIGHTGRAY, CONTINUE, " %02X", *(b + i) );
	}

	mprintf( LIGHTGRAY, CONTINUE, "\r\n" );
}



/*********															*********/
/*																			*/
/*	User interface (to show the printer job status) function				*/
/*																			*/
/*		This functions is called from "ui_status_monitor()" in "ui.c" as 		*/
/*		callback function. 													*/
/*		This routine will be installed by "printer_init_commands()"	at		*/
/*		printer device enumeration.											*/
/*																			*/
/*********															*********/

void strg_status_monitor( unsigned char dummy_for_compatibility )
{
	storage_instance		*si_ptr;
	volume_param			*vpp;
	unsigned long			total_size;
	
	dummy_for_compatibility	= dummy_for_compatibility;
	
	if ( NULL == g_storage_device_ptr )
		return;
	
	si_ptr		= (storage_instance *)(g_storage_device_ptr->class_instance_ptr);
	
	if ( si_ptr->state != si_ptr->previous_state )
	{
		mprintf( BG_BLUE | WHITE    , CONTINUE, "    Storage" );
		mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "(addr %2d) : ", g_storage_device_ptr->address );

		switch ( si_ptr->state )
		{
			case IDLE :
				mprintf( BG_BLUE | LIGHTGRAY , CONTINUE, "idle... " );
				mprintf( BG_BLUE | YELLOW    , CONTINUE, "type [s] to proceed mount after media insert" );
				break;
			case WAIT_FOR_CARD :
			case WAIT_STABILIZED :
				mprintf( BG_BLUE | LIGHTRED  , CONTINUE, "%-55s", "waiting for media" );
				break;
			case MOUNT_PROCEED :
				mprintf( BG_BLUE | LIGHTRED  , CONTINUE, "%-55s", "mount processing" );
				break;
			case MOUNTED :
				{
					vpp		= (si_ptr->volume_instance)[ 0 ];
					total_size	= vpp->total_sectors * vpp->bytes_per_sector;

					mprintf( BG_BLUE | WHITE,     CONTINUE, "mounted" );
					mprintf( BG_BLUE | LIGHTGRAY, CONTINUE, "   media size = %.2lf MB            ", (double)total_size / (double)(1L << 20) );
				}
				break;
			case UNMOUNT_PROCEED :
				mprintf( BG_BLUE | LIGHTRED  , CONTINUE, "%-40s", "unmount processing" );
				break;
		}

		si_ptr->previous_state	= si_ptr->state;
	}
}





